"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _sequelize = require("sequelize");

var _AggregateIdentifier = _interopRequireDefault(require("../../common/AggregateIdentifier"));

var _TmdbSeriesIdentifier = _interopRequireDefault(require("./identifiers/TmdbSeriesIdentifier"));

var _TmdbEpisodeIdentifier = _interopRequireDefault(require("./identifiers/TmdbEpisodeIdentifier"));

var _TvdbSeriesIdentifier = _interopRequireDefault(require("./identifiers/TvdbSeriesIdentifier"));

var _TvdbEpisodeIdentifier = _interopRequireDefault(require("./identifiers/TvdbEpisodeIdentifier"));

var _series = require("../../../models/series");

var _episode = require("../../../models/episode");

var _IdentificationError = _interopRequireDefault(require("../../errors/IdentificationError"));

var _logger = _interopRequireDefault(require("../../../submodules/logger"));

var _guessit = _interopRequireDefault(require("../../../submodules/guessit"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class SeriesIndexer {
  /**
   *
   * @param {Oblecto} oblecto
   */
  constructor(oblecto) {
    this.oblecto = oblecto;
    this.seriesIdentifier = new _AggregateIdentifier.default();
    this.episodeIdentifer = new _AggregateIdentifier.default();
    const seriesIdentifiers = {
      'tmdb': _TmdbSeriesIdentifier.default,
      'tvdb': _TvdbSeriesIdentifier.default
    };
    const episodeIdentifiers = {
      'tmdb': _TmdbEpisodeIdentifier.default,
      'tvdb': _TvdbEpisodeIdentifier.default
    };

    for (let identifier of this.oblecto.config.tvshows.seriesIdentifiers) {
      _logger.default.log('DEBUG', `Loading ${identifier} series identifier`);

      this.seriesIdentifier.loadIdentifier(new seriesIdentifiers[identifier](this.oblecto));
    }

    for (let identifier of this.oblecto.config.tvshows.episodeIdentifiers) {
      _logger.default.log('DEBUG', `Loading ${identifier} episode identifier`);

      this.episodeIdentifer.loadIdentifier(new episodeIdentifiers[identifier](this.oblecto));
    } // Register task availability to Oblecto queue


    this.oblecto.queue.registerJob('indexEpisode', async job => await this.indexFile(job.path));
  }

  async indexSeries(file, guessitIdentification) {
    let seriesIdentification;

    try {
      seriesIdentification = await this.seriesIdentifier.identify(file.path, guessitIdentification);
    } catch (e) {
      throw new _IdentificationError.default(`Could not identify series of ${file.path}`);
    }

    _logger.default.log('DEBUG', `${file.path} series identified: ${seriesIdentification.seriesName}`);

    const identifiers = ['tvdbid', 'tmdbid'];
    let seriesQuery = [];

    for (let identifier of identifiers) {
      if (!seriesIdentification[identifier]) continue;
      seriesQuery.push({
        [identifier]: seriesIdentification[identifier]
      });
    }

    let [series, seriesCreated] = await _series.Series.findOrCreate({
      where: {
        [_sequelize.Op.or]: seriesQuery
      },
      defaults: seriesIdentification
    });

    if (seriesCreated) {
      this.oblecto.queue.pushJob('updateSeries', series);
      this.oblecto.queue.queueJob('downloadSeriesPoster', series);
    }

    return series;
  }

  async indexFile(episodePath) {
    let file = await this.oblecto.fileIndexer.indexVideoFile(episodePath);
    const guessitIdentification = await _guessit.default.identify(episodePath); // Some single season shows usually don't have a season in the title,
    // therefore whe should set it to 1 by default.

    if (!guessitIdentification.season) {
      guessitIdentification.season = 1;
    }

    let series = await this.indexSeries(file, guessitIdentification);
    let episodeIdentification;

    try {
      episodeIdentification = await this.episodeIdentifer.identify(episodePath, guessitIdentification, series);
    } catch (e) {
      throw new _IdentificationError.default(`Could not identify episode ${episodePath}`);
    }

    _logger.default.log('DEBUG', `${file.path} episode identified ${episodeIdentification.episodeName}`);

    let [episode, episodeCreated] = await _episode.Episode.findOrCreate({
      where: {
        airedSeason: episodeIdentification.airedSeason || 1,
        airedEpisodeNumber: episodeIdentification.airedEpisodeNumber,
        SeriesId: series.id
      },
      defaults: episodeIdentification
    });
    await episode.addFile(file);

    if (episodeCreated) {
      this.oblecto.queue.pushJob('updateEpisode', episode);
      this.oblecto.queue.queueJob('downloadEpisodeBanner', episode);
    }
  }

}

exports.default = SeriesIndexer;