"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _jsonwebtoken = _interopRequireDefault(require("jsonwebtoken"));

var _events = require("events");

var _trackEpisode = require("../../models/trackEpisode");

var _trackMovie = require("../../models/trackMovie");

var _logger = _interopRequireDefault(require("../../submodules/logger"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class RealtimeClient extends _events.EventEmitter {
  /**
   * @param {Oblecto} oblecto
   * @param {*} socket
   */
  constructor(oblecto, socket) {
    super();
    this.clientName = 'default';
    this.oblecto = oblecto;
    this.socket = socket;
    this.user = null;
    this.storage = {
      series: {},
      movie: {}
    };
    this.socket.on('authenticate', data => this.authenticationHandler(data));
    this.socket.on('playing', data => this.playingHandler(data));
    this.socket.on('disconnect', () => this.disconnectHandler());
    setInterval(() => {
      this.saveAllTracks();
    }, 10000);
  }

  authenticationHandler(data) {
    try {
      this.user = _jsonwebtoken.default.verify(data.token, this.oblecto.config.authentication.secret);
    } catch (e) {
      _logger.default.log('WARN', 'An unauthorized user attempted connection to realtime server');

      _logger.default.log('WARN', 'Disconnecting client...');

      this.socket.disconnect();
    }
  }

  playingHandler(data) {
    if (this.user == null) return;
    if (data.type === 'tv') return this.playingEpisodeHandler(data);
    if (data.type === 'movie') return this.playingMovieHandler(data);
  }

  playingEpisodeHandler(data) {
    this.storage.series[data.episodeId] = data;
  }

  playingMovieHandler(data) {
    this.storage.movie[data.movieId] = data;
  }

  disconnectHandler() {
    this.emit('disconnect');
  }

  async saveEpisodeTrack(id) {
    if (this.user === null) return;
    let [item, created] = await _trackEpisode.TrackEpisode.findOrCreate({
      where: {
        UserId: this.user.id,
        EpisodeId: id
      },
      defaults: {
        time: this.storage.series[id].time,
        progress: this.storage.series[id].progress
      }
    });
    if (created) return;
    await item.update({
      time: this.storage.series[id].time,
      progress: this.storage.series[id].progress
    });
    delete this.storage.series[id];
  }

  async saveMovieTrack(id) {
    if (this.user == null) return;
    let [item, created] = await _trackMovie.TrackMovie.findOrCreate({
      where: {
        UserId: this.user.id,
        MovieId: id
      },
      defaults: {
        time: this.storage.movie[id].time,
        progress: this.storage.movie[id].progress
      }
    });
    if (created) return;
    await item.update({
      time: this.storage.movie[id].time,
      progress: this.storage.movie[id].progress
    });
    delete this.storage.movie[id];
  }

  async saveAllTracks() {
    for (let i in this.storage.series) {
      await this.saveEpisodeTrack(i);
    }

    for (let i in this.storage.movie) {
      await this.saveMovieTrack(i);
    }
  }

  async playEpisode(episodeId) {
    this.socket.emit('play', {
      episodeId
    });
  }

  async playMovie(movieId) {
    this.socket.emit('play', {
      movieId
    });
  }

}

exports.default = RealtimeClient;