"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _events = require("events");

var _uuid = require("uuid");

var _logger = _interopRequireDefault(require("../../submodules/logger"));

var _stream = _interopRequireDefault(require("stream"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class StreamSession extends _events.EventEmitter {
  /**
   *
   * @param {File} file
   * @param options
   * @param {Oblecto} oblecto
   */
  constructor(file, options, oblecto) {
    super();

    _logger.default.log('INFO', 'New StreamSession initiating');

    this.oblecto = oblecto;
    this.file = file;
    this.sessionId = (0, _uuid.v4)();
    this.destinations = [];
    this.targetFormats = options.target.formats;
    this.targetVideoCodecs = options.target.videoCodecs;
    this.targetAudioCodecs = options.target.audioCodecs;
    this.format = this.targetFormats[0] || 'mp4';
    this.videoCodec = this.targetVideoCodecs[0] || 'h264';
    this.audioCodec = this.targetAudioCodecs[0] || 'aac';
    this.offset = options.offset || 0;
    this.inputStream = new _stream.default.PassThrough();
    this.outputStream = new _stream.default.PassThrough();
    this.outputStream.on('pause', () => this.outputPause());
    this.outputStream.on('resume', () => this.outputResume());
    this.outputStream.on('close', () => {
      _logger.default.log('INFO', this.sessionId, 'output stream has closed');

      this.endSession();
    });
    this.httpHeaders = {
      'Content-Type': this.getOutputMimetype(),
      'Accept-Ranges': 'none'
    };
    this.httpStatusCode = 200;
    this.timeoutTime = 10000;
    this.startTimeout();
  }

  outputPause() {}

  outputResume() {}

  endSession() {
    if (this.process) {
      this.process.kill();
    }

    this.emit('close');
  }

  startTimeout() {
    if (this.timeout) this.clearTimeout();
    this.timeout = setTimeout(() => {
      _logger.default.log('INFO', 'StreamSession', this.sessionId, 'has timed out. Destroying output stream');

      this.outputStream.destroy();
    }, this.timeoutTime);
  }

  async addDestination(destination) {
    this.destinations.push(destination);
    this.outputStream.pipe(destination.stream);

    if (destination.type === 'http') {
      destination.stream.writeHead(this.httpStatusCode, this.httpHeaders);
    }

    let _this = this;

    destination.stream.on('error', err => {
      _logger.default.log('ERROR', this.sessionId, err);
    }).on('close', function () {
      for (let i in _this.destinations) {
        if (_this.destinations[i].stream === this) {
          _this.destinations.splice(i, 1);
        }
      }

      if (_this.destinations.length === 0) {
        _logger.default.log('INFO', _this.sessionId, 'last client has disconnected. Setting timeout output stream');

        _this.startTimeout();
      }
    });
  }

  clearTimeout() {
    clearTimeout(this.timeout);
  }

  async startStream() {
    this.clearTimeout();
  }

  getOutputMimetype() {
    if (this.format === 'mp4') return 'video/mp4';
    if (this.format === 'mpegts') return 'video/mp2t';
    if (this.format === 'matroska') return 'video/x-matroska';
    return 'video';
  }

  getFfmpegVideoCodec() {
    if (this.oblecto.config.transcoding.hardwareAcceleration) {
      if (this.oblecto.config.transcoding.hardwareAccelerator === 'cuda') {
        if (this.videoCodec === 'h264') {
          return 'h264_nvenc';
        }
      }

      if (this.oblecto.config.transcoding.hardwareAccelerator === 'vaapi') {
        if (this.videoCodec === 'h264') {
          return 'h264_vaapi';
        }
      }
    }

    let codecs = {
      'h264': 'libx264'
    };
    if (codecs[this.videoCodec]) return codecs[this.videoCodec];
    return this.videoCodec;
  }

}

exports.default = StreamSession;