"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _sequelize = _interopRequireDefault(require("sequelize"));

var _fs = _interopRequireDefault(require("fs"));

var _restifyErrors = _interopRequireDefault(require("restify-errors"));

var _sharp = _interopRequireDefault(require("sharp"));

var _auth = _interopRequireDefault(require("../middleware/auth"));

var _episode = require("../../../models/episode");

var _series = require("../../../models/series");

var _trackEpisode = require("../../../models/trackEpisode");

var _file = require("../../../models/file");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const Op = _sequelize.default.Op;

var _default = (server, oblecto) => {
  // Endpoint to get a list of episodes from all series
  server.get('/episodes/list/:sorting', _auth.default.requiresAuth, async function (req, res, next) {
    let limit = 20;
    let page = 0;
    let AllowedOrders = ['desc', 'asc'];
    if (AllowedOrders.indexOf(req.params.order.toLowerCase()) === -1) return next(new _restifyErrors.default.BadRequestError('Sorting order is invalid'));
    if (!(req.params.sorting in _episode.Episode.rawAttributes)) return next(new _restifyErrors.default.BadRequestError('Sorting method is invalid'));
    if (req.params.count && Number.isInteger(req.params.count)) limit = parseInt(req.params.count);
    if (req.params.page && Number.isInteger(req.params.page)) page = parseInt(req.params.page);
    let results = await _episode.Episode.findAll({
      include: [_series.Series, {
        model: _trackEpisode.TrackEpisode,
        required: false,
        where: {
          userId: req.authorization.user.id
        }
      }],
      order: [[req.params.sorting, req.params.order]],
      limit,
      offset: limit * page
    });
    res.send(results);
  }); // Endpoint to get a banner image for an episode based on the local episode ID

  server.get('/episode/:id/banner', async function (req, res, next) {
    // Get episode data
    let episode;

    try {
      episode = await _episode.Episode.findByPk(req.params.id, {
        include: [_file.File]
      });
    } catch (e) {
      return next(new _restifyErrors.default.NotFoundError('Episode not found'));
    }

    let imagePath = oblecto.artworkUtils.episodeBannerPath(episode, req.params.size || 'medium');

    _fs.default.createReadStream(imagePath).on('error', () => {
      return next(new _restifyErrors.default.NotFoundError('No banner found'));
    }).pipe(res);
  });
  server.put('/episode/:id/banner', _auth.default.requiresAuth, async function (req, res, next) {
    let episode = await _episode.Episode.findByPk(req.params.id, {
      include: [_file.File]
    });

    if (!episode) {
      return next(new _restifyErrors.default.NotFoundError('Episode does not exist'));
    }

    let thumbnailPath = this.oblecto.artworkUtils.episodeBannerPath(episode);

    if (req.files.length < 1) {
      return next(new _restifyErrors.default.MissingParameter('Image file is missing'));
    }

    let uploadPath = req.files[Object.keys(req.files)[0]].path;

    try {
      let image = await (0, _sharp.default)(uploadPath);
      let metadata = await image.metadata();
      let ratio = metadata.height / metadata.width;

      if (!(1 <= ratio <= 2)) {
        return next(new _restifyErrors.default.InvalidContent('Image aspect ratio is incorrect'));
      }
    } catch (e) {
      return next(new _restifyErrors.default.InvalidContent('File is not an image'));
    }

    try {
      _fs.default.copyFile(uploadPath, thumbnailPath, err => {
        if (err) throw err;

        for (let size of Object.keys(this.oblecto.config.artwork.poster)) {
          this.oblecto.queue.pushJob('rescaleImage', {
            from: this.oblecto.artworkUtils.episodeBannerPath(episode),
            to: this.oblecto.artworkUtils.episodeBannerPath(episode, size),
            width: this.oblecto.config.artwork.poster[size]
          });
        }

        res.send(['success']);
      });
    } catch (e) {
      console.log(e);
      return next(new _restifyErrors.default.Internal('An error has occured during upload of banner'));
    }

    next();
  }); // Endpoint to list all stored files for the specific episode

  server.get('/episode/:id/files', _auth.default.requiresAuth, async function (req, res) {
    let episode = await _episode.Episode.findByPk(req.params.id, {
      include: [_file.File]
    });
    res.send(episode.files);
  }); // Endpoint to send episode video file to the client
  // TODO: move this to the file route and use file id to play, abstracting this from episodes

  server.get('/episode/:id/play', async function (req, res, next) {
    // search for attributes
    let episode = await _episode.Episode.findByPk(req.params.id, {
      include: [_file.File]
    });
    let file = episode.files[0];
    res.redirect(`/stream/${file.id}`, next);
  }); // Endpoint to retrieve episode details based on the local episode ID

  server.get('/episode/:id/info', _auth.default.requiresAuth, async function (req, res) {
    // search for attributes
    let episode = await _episode.Episode.findByPk(req.params.id, {
      include: [_file.File, _series.Series, {
        model: _trackEpisode.TrackEpisode,
        required: false,
        where: {
          userId: req.authorization.user.id
        }
      }]
    });
    res.send(episode);
  }); // Endpoint to retrieve the episode next in series based on the local episode ID

  server.get('/episode/:id/next', _auth.default.requiresAuth, async function (req, res) {
    // search for attributes
    let results = await _episode.Episode.findByPk(req.params.id);
    let episode = await _episode.Episode.findOne({
      where: {
        SeriesId: results.SeriesId,
        [Op.or]: [{
          [Op.and]: [{
            airedEpisodeNumber: {
              [Op.gt]: results.airedEpisodeNumber
            }
          }, {
            airedSeason: {
              [Op.gte]: results.airedSeason
            }
          }]
        }, {
          [Op.and]: [{
            airedSeason: {
              [Op.gt]: results.airedSeason
            }
          }]
        }]
      },
      order: [['airedSeason', 'ASC'], ['airedEpisodeNumber', 'ASC']]
    });
    res.send(episode);
  });
  server.get('/episodes/search/:name', _auth.default.requiresAuth, async function (req, res) {
    // search for attributes
    let episode = await _episode.Episode.findAll({
      where: {
        episodeName: {
          [Op.like]: '%' + req.params.name + '%'
        }
      },
      include: [_file.File, _series.Series, {
        model: _trackEpisode.TrackEpisode,
        required: false,
        where: {
          userId: req.authorization.user.id
        }
      }]
    });
    res.send(episode);
  }); // Endpoint to get the episodes currently being watched

  server.get('/episodes/watching', _auth.default.requiresAuth, async function (req, res) {
    // search for attributes
    let watching = await _episode.Episode.findAll({
      include: [_series.Series, {
        model: _trackEpisode.TrackEpisode,
        required: true,
        where: {
          userId: req.authorization.user.id,
          progress: {
            [_sequelize.default.Op.lt]: 0.9
          },
          updatedAt: {
            [_sequelize.default.Op.gt]: new Date() - 1000 * 60 * 60 * 24 * 7
          }
        }
      }],
      order: [['updatedAt', 'DESC']]
    }); // We are only interested in the episode objects, so extract all the episode object from
    // each track object and send the final mapped array to the client

    res.send(watching);
  });
  server.get('/episodes/next', _auth.default.requiresAuth, async function (req, res, next) {
    // Next episodes currently doesn't work on sqlite as the LPAD function doesn't exist
    // Todo: Fix next episodes endpoint to support sqlite
    if (oblecto.config.database.dialect === 'sqlite') return next(new _restifyErrors.default.NotImplementedError('Next episode is not supported when using sqlite (yet)')); // search for attributes

    let latestWatched = await _episode.Episode.findAll({
      attributes: {
        include: [[_sequelize.default.fn('MAX', _sequelize.default.col('absoluteNumber')), 'absoluteNumber'], [_sequelize.default.fn('MAX', _sequelize.default.fn('concat', _sequelize.default.fn('LPAD', _sequelize.default.col('airedSeason'), 2, '0'), _sequelize.default.fn('LPAD', _sequelize.default.col('airedEpisodeNumber'), 2, '0'))), 'seasonepisode'], [_sequelize.default.fn('MAX', _sequelize.default.col('firstAired')), 'firstAired']]
      },
      include: [{
        model: _trackEpisode.TrackEpisode,
        required: true,
        where: {
          userId: req.authorization.user.id,
          progress: {
            [_sequelize.default.Op.gt]: 0.9
          },
          updatedAt: {
            [_sequelize.default.Op.gt]: new Date() - 1000 * 60 * 60 * 24 * 7
          }
        }
      }],
      group: ['SeriesId']
    });
    let nextUp = [];

    for (let latest of latestWatched) {
      latest = latest.toJSON();
      let next = await _episode.Episode.findOne({
        attributes: {
          include: [[_sequelize.default.fn('concat', _sequelize.default.fn('LPAD', _sequelize.default.col('airedSeason'), 2, '0'), _sequelize.default.fn('LPAD', _sequelize.default.col('airedEpisodeNumber'), 2, '0')), 'seasonepisode']]
        },
        include: [_series.Series, {
          model: _trackEpisode.TrackEpisode,
          where: {
            userId: req.authorization.user.id
          }
        }],
        where: _sequelize.default.and(_sequelize.default.where(_sequelize.default.col('SeriesId'), '=', latest.SeriesId), _sequelize.default.where(_sequelize.default.fn('concat', _sequelize.default.fn('LPAD', _sequelize.default.col('airedSeason'), 2, '0'), _sequelize.default.fn('LPAD', _sequelize.default.col('airedEpisodeNumber'), 2, '0')), '>', latest.seasonepisode)),
        order: [_sequelize.default.col('seasonepisode')]
      });

      if (next) {
        nextUp.push(next);
      }
    }

    res.send(nextUp);
  });
};

exports.default = _default;